import os
from functools import cached_property

from utils import File, Log

log = Log("AbstractDocTextMixin")


class AbstractDocTextMixin:

    @cached_property
    def text_from_metadata(self) -> str:
        return "\n".join([self.description])

    @cached_property
    def text_path(self) -> str:
        return os.path.join(self.dir_doc, "doc.txt")

    @property
    def has_text(self) -> bool:
        return os.path.exists(self.text_path)

    @cached_property
    def doc_readme_path(self) -> str:
        return os.path.join(self.dir_doc, "README.md")

    def extract_text(self):
        if not self.has_text:
            File(self.text_path).write(self.text_from_metadata)
            log.info(f"Wrote {self.text_path}")

    def get_text(self):
        if not os.path.exists(self.text_path):
            return ""
        return File(self.text_path).read()



import os
from dataclasses import dataclass
from functools import cached_property

import pandas as pd
from utils import WWW, File, Log

from scraper.abstract_doc.AbstractDoc import AbstractDoc

log = Log("AbstractExcelSpreadsheet")


@dataclass
class AbstractExcelSpreadsheet(AbstractDoc):
    url_excel: str

    @cached_property
    def excel_path(self) -> str:
        return os.path.join(self.dir_doc, "doc.xlsx")

    @property
    def has_excel(self) -> bool:
        return os.path.exists(self.excel_path)

    def download_excel(self):
        WWW(self.url_excel).download_binary(self.excel_path)

    # ----------------------------------------------------------------
    # Worksheets (extracted from Excel)
    # ----------------------------------------------------------------
    @property
    def dir_worksheets(self) -> str:
        return os.path.join(self.dir_doc, "worksheets")

    def extract_worksheets(self):
        if not os.path.exists(self.excel_path):
            return
        excel = pd.ExcelFile(self.excel_path)
        for i_sheet, sheet_name in enumerate(excel.sheet_names, 1):
            sheet_name_cleaned = sheet_name.replace("/", "_").replace(
                " ", "_"
            )
            csv_path = os.path.join(
                self.dir_worksheets,
                f"{i_sheet:02d}-{sheet_name_cleaned}.csv",
            )
            if os.path.exists(csv_path):
                continue

            df = excel.parse(sheet_name)
            os.makedirs(self.dir_worksheets, exist_ok=True)
            df.to_csv(csv_path, index=False)
            log.info(f"Wrote {csv_path}")

    # ----------------------------------------------------------------
    # Text (extracted from Worksheet CSVs)
    # ----------------------------------------------------------------

    def extract_text(self):
        if os.path.exists(self.text_path):
            return
        if not os.path.exists(self.dir_worksheets):
            return

        content_groups = []
        for file_name in sorted(os.listdir(self.dir_worksheets)):
            if not file_name.endswith(".csv"):
                continue
            csv_path = os.path.join(self.dir_worksheets, file_name)
            content = File(csv_path).read()
            content_groups.append(f"--- {file_name} ---\n")
            content_groups.append(content)

        text = "\n".join(content_groups)
        File(self.text_path).write(text)
        log.info(f"Wrote {self.text_path}")

    # ----------------------------------------------------------------
    def scrape_extended_data_for_doc(self):
        if not self.has_excel:
            self.download_excel()

        if not os.path.exists(self.dir_worksheets):
            self.extract_worksheets()

        if not self.has_text:
            self.extract_text()



import os
import tempfile
from abc import ABC
from dataclasses import dataclass
from functools import cached_property

from utils import WWW, File, JSONFile, Log, PDFFile

from scraper.abstract_doc import AbstractDoc

log = Log("AbstractPDFDoc")


@dataclass
class AbstractPDFDoc(AbstractDoc, ABC):
    url_pdf: str

    # PDF
    # ----------------------------------------------------------------
    @cached_property
    def pdf_path(self) -> str:
        return os.path.join(self.dir_doc, "doc.pdf")

    @property
    def has_pdf(self) -> bool:
        return os.path.exists(self.pdf_path)

    def download_pdf(self):
        with tempfile.NamedTemporaryFile(
            suffix=".pdf", delete=False
        ) as temp_pdf_f:
            temp_pdf_path = temp_pdf_f.name
            WWW(self.url_pdf).download_binary(temp_pdf_path)
            PDFFile(temp_pdf_path).compress(self.pdf_path)

    # Blocks (Extracted from PDF)
    # ----------------------------------------------------------------
    @cached_property
    def blocks_path(self) -> str:
        return os.path.join(self.dir_doc, "blocks.json")

    def extract_blocks(self):
        if not os.path.exists(self.pdf_path):
            return
        pdf_file = PDFFile(self.pdf_path)
        blocks = pdf_file.get_blocks()
        JSONFile(self.blocks_path).write(blocks)
        log.info(f"Wrote {self.blocks_path} ({len(blocks):,} blocks)")

        text_lines = [block["text"] for block in blocks if block["text"]]
        File(self.doc_readme_path).write("\n\n".join(text_lines))
        log.info(f"Wrote {self.doc_readme_path}")

    def get_blocks(self):
        if not os.path.exists(self.blocks_path):
            return []
        return JSONFile(self.blocks_path).read()

    def extract_text(self):  # overrides AbstractDocTextMixin
        blocks = self.get_blocks()
        text_list = [block["text"] for block in blocks if block["text"]]
        content = "\n\n".join(text_list)
        File(self.text_path).write(content)
        log.info(f"Wrote {self.text_path}")
        File(self.doc_readme_path).write(content)
        log.info(f"Wrote {self.doc_readme_path}")

    # All
    # ----------------------------------------------------------------
    def scrape_extended_data_for_doc(self):
        if not self.has_pdf:
            self.download_pdf()

        if self.has_pdf:
            if not os.path.exists(self.blocks_path):
                self.extract_blocks()

            if os.path.exists(self.blocks_path) and not os.path.exists(
                self.text_path
            ):
                self.extract_text()
        else:
            # HACK cleanup
            if os.path.exists(self.blocks_path):
                os.remove(self.blocks_path)
            if os.path.exists(self.text_path):
                os.remove(self.text_path)
